# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.segment_delete_response import SegmentDeleteResponse
from ....types.segment_update_response import SegmentUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSegmentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def update(
        self,
        dubbing_id: str,
        segment_id: str,
        language: str,
        *,
        start_time: typing.Optional[float] = OMIT,
        end_time: typing.Optional[float] = OMIT,
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SegmentUpdateResponse]:
        """
        Modifies a single segment with new text and/or start/end times. Will update the values for only a specific language of a segment. Does not automatically regenerate the dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        segment_id : str
            ID of the segment

        language : str
            ID of the language.

        start_time : typing.Optional[float]

        end_time : typing.Optional[float]

        text : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SegmentUpdateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/segment/{jsonable_encoder(segment_id)}/{jsonable_encoder(language)}",
            method="PATCH",
            json={
                "start_time": start_time,
                "end_time": end_time,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SegmentUpdateResponse,
                    construct_type(
                        type_=SegmentUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, dubbing_id: str, segment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SegmentDeleteResponse]:
        """
        Deletes a single segment from the dubbing.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        segment_id : str
            ID of the segment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SegmentDeleteResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/segment/{jsonable_encoder(segment_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SegmentDeleteResponse,
                    construct_type(
                        type_=SegmentDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSegmentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def update(
        self,
        dubbing_id: str,
        segment_id: str,
        language: str,
        *,
        start_time: typing.Optional[float] = OMIT,
        end_time: typing.Optional[float] = OMIT,
        text: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SegmentUpdateResponse]:
        """
        Modifies a single segment with new text and/or start/end times. Will update the values for only a specific language of a segment. Does not automatically regenerate the dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        segment_id : str
            ID of the segment

        language : str
            ID of the language.

        start_time : typing.Optional[float]

        end_time : typing.Optional[float]

        text : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SegmentUpdateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/segment/{jsonable_encoder(segment_id)}/{jsonable_encoder(language)}",
            method="PATCH",
            json={
                "start_time": start_time,
                "end_time": end_time,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SegmentUpdateResponse,
                    construct_type(
                        type_=SegmentUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, dubbing_id: str, segment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SegmentDeleteResponse]:
        """
        Deletes a single segment from the dubbing.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        segment_id : str
            ID of the segment

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SegmentDeleteResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/dubbing/resource/{jsonable_encoder(dubbing_id)}/segment/{jsonable_encoder(segment_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SegmentDeleteResponse,
                    construct_type(
                        type_=SegmentDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_token_purpose import ConversationTokenPurpose


class ConversationTokenResponseModel(UncheckedBaseModel):
    agent_id: str = pydantic.Field()
    """
    The ID of the agent
    """

    conversation_token: str = pydantic.Field()
    """
    The token for the agent
    """

    expiration_time_unix_secs: typing.Optional[int] = pydantic.Field(default=None)
    """
    The expiration time of the token in unix seconds
    """

    conversation_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the conversation
    """

    purpose: ConversationTokenPurpose = pydantic.Field()
    """
    The purpose of the token
    """

    token_requester_user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user ID of the entity who requested the token
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.close_context import CloseContext
from ...types.close_socket import CloseSocket
from ...types.flush_context import FlushContext
from ...types.initialise_context import InitialiseContext
from ...types.initialize_connection_multi import InitializeConnectionMulti
from ...types.keep_context_alive import KeepContextAlive
from ...types.send_text_multi import SendTextMulti

SendMessageMulti = typing.Union[
    InitializeConnectionMulti,
    InitialiseContext,
    SendTextMulti,
    FlushContext,
    CloseContext,
    CloseSocket,
    KeepContextAlive,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.dubbing_transcripts_response_model import DubbingTranscriptsResponseModel
from .types.transcripts_get_request_format_type import TranscriptsGetRequestFormatType


class RawTranscriptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        dubbing_id: str,
        language_code: str,
        format_type: TranscriptsGetRequestFormatType,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DubbingTranscriptsResponseModel]:
        """
        Fetch the transcript for one of the languages in a dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ISO-693 language code to retrieve the transcript for. Use 'source' to fetch the transcript of the original media.

        format_type : TranscriptsGetRequestFormatType
            Format to return transcript in. For subtitles use either 'srt' or 'webvtt', and for a full transcript use 'json'. The 'json' format is not yet supported for Dubbing Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DubbingTranscriptsResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/dubbing/{jsonable_encoder(dubbing_id)}/transcripts/{jsonable_encoder(language_code)}/format/{jsonable_encoder(format_type)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DubbingTranscriptsResponseModel,
                    construct_type(
                        type_=DubbingTranscriptsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTranscriptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        dubbing_id: str,
        language_code: str,
        format_type: TranscriptsGetRequestFormatType,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DubbingTranscriptsResponseModel]:
        """
        Fetch the transcript for one of the languages in a dub.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        language_code : str
            ISO-693 language code to retrieve the transcript for. Use 'source' to fetch the transcript of the original media.

        format_type : TranscriptsGetRequestFormatType
            Format to return transcript in. For subtitles use either 'srt' or 'webvtt', and for a full transcript use 'json'. The 'json' format is not yet supported for Dubbing Studio.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DubbingTranscriptsResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/dubbing/{jsonable_encoder(dubbing_id)}/transcripts/{jsonable_encoder(language_code)}/format/{jsonable_encoder(format_type)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DubbingTranscriptsResponseModel,
                    construct_type(
                        type_=DubbingTranscriptsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .speaker_response_model import SpeakerResponseModel
from .speaker_separation_response_model_status import SpeakerSeparationResponseModelStatus


class SpeakerSeparationResponseModel(UncheckedBaseModel):
    voice_id: str = pydantic.Field()
    """
    The ID of the voice.
    """

    sample_id: str = pydantic.Field()
    """
    The ID of the sample.
    """

    status: SpeakerSeparationResponseModelStatus = pydantic.Field()
    """
    The status of the speaker separation.
    """

    speakers: typing.Optional[typing.Dict[str, typing.Optional[SpeakerResponseModel]]] = pydantic.Field(default=None)
    """
    The speakers of the sample.
    """

    selected_speaker_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the selected speakers.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .required_constraints import RequiredConstraints


class ObjectJsonSchemaPropertyOutput(UncheckedBaseModel):
    type: typing.Optional[typing.Literal["object"]] = None
    required: typing.Optional[typing.List[str]] = None
    description: typing.Optional[str] = None
    properties: typing.Optional[typing.Dict[str, "ObjectJsonSchemaPropertyOutputPropertiesValue"]] = None
    required_constraints: typing.Optional[RequiredConstraints] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .array_json_schema_property_output import ArrayJsonSchemaPropertyOutput  # noqa: E402, I001
from .object_json_schema_property_output_properties_value import ObjectJsonSchemaPropertyOutputPropertiesValue  # noqa: E402, I001

update_forward_refs(ObjectJsonSchemaPropertyOutput, ArrayJsonSchemaPropertyOutput=ArrayJsonSchemaPropertyOutput)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .list_response_meta import ListResponseMeta
from .test_invocation_summary_response_model import TestInvocationSummaryResponseModel


class GetTestInvocationsPageResponseModel(UncheckedBaseModel):
    meta: typing.Optional[ListResponseMeta] = None
    results: typing.List[TestInvocationSummaryResponseModel]
    next_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor for the next page of results
    """

    has_more: bool = pydantic.Field()
    """
    Whether there are more results available
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.seat_type import SeatType
from ...types.update_workspace_member_response_model import UpdateWorkspaceMemberResponseModel
from .raw_client import AsyncRawMembersClient, RawMembersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MembersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMembersClient
        """
        return self._raw_client

    def update(
        self,
        *,
        email: str,
        is_locked: typing.Optional[bool] = OMIT,
        workspace_role: typing.Optional[SeatType] = OMIT,
        workspace_seat_type: typing.Optional[SeatType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWorkspaceMemberResponseModel:
        """
        Updates attributes of a workspace member. Apart from the email identifier, all parameters will remain unchanged unless specified. This endpoint may only be called by workspace administrators.

        Parameters
        ----------
        email : str
            Email of the target user.

        is_locked : typing.Optional[bool]
            Whether to lock or unlock the user account.

        workspace_role : typing.Optional[SeatType]
            The workspace role of the user. This is deprecated, use `workspace_seat_type` instead.

        workspace_seat_type : typing.Optional[SeatType]
            The workspace seat type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWorkspaceMemberResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.members.update(
            email="email",
        )
        """
        _response = self._raw_client.update(
            email=email,
            is_locked=is_locked,
            workspace_role=workspace_role,
            workspace_seat_type=workspace_seat_type,
            request_options=request_options,
        )
        return _response.data


class AsyncMembersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMembersClient
        """
        return self._raw_client

    async def update(
        self,
        *,
        email: str,
        is_locked: typing.Optional[bool] = OMIT,
        workspace_role: typing.Optional[SeatType] = OMIT,
        workspace_seat_type: typing.Optional[SeatType] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWorkspaceMemberResponseModel:
        """
        Updates attributes of a workspace member. Apart from the email identifier, all parameters will remain unchanged unless specified. This endpoint may only be called by workspace administrators.

        Parameters
        ----------
        email : str
            Email of the target user.

        is_locked : typing.Optional[bool]
            Whether to lock or unlock the user account.

        workspace_role : typing.Optional[SeatType]
            The workspace role of the user. This is deprecated, use `workspace_seat_type` instead.

        workspace_seat_type : typing.Optional[SeatType]
            The workspace seat type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWorkspaceMemberResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.members.update(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            email=email,
            is_locked=is_locked,
            workspace_role=workspace_role,
            workspace_seat_type=workspace_seat_type,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.add_voice_response_model import AddVoiceResponseModel
from ...types.start_pvc_voice_training_response_model import StartPvcVoiceTrainingResponseModel
from .raw_client import AsyncRawPvcClient, RawPvcClient

if typing.TYPE_CHECKING:
    from .samples.client import AsyncSamplesClient, SamplesClient
    from .verification.client import AsyncVerificationClient, VerificationClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PvcClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPvcClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._samples: typing.Optional[SamplesClient] = None
        self._verification: typing.Optional[VerificationClient] = None

    @property
    def with_raw_response(self) -> RawPvcClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPvcClient
        """
        return self._raw_client

    def create(
        self,
        *,
        name: str,
        language: str,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddVoiceResponseModel:
        """
        Creates a new PVC voice with metadata but no samples

        Parameters
        ----------
        name : str
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : str
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddVoiceResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.create(
            name="John Smith",
            language="en",
        )
        """
        _response = self._raw_client.create(
            name=name, language=language, description=description, labels=labels, request_options=request_options
        )
        return _response.data

    def update(
        self,
        voice_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddVoiceResponseModel:
        """
        Edit PVC voice metadata

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        name : typing.Optional[str]
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : typing.Optional[str]
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddVoiceResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.update(
            voice_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.update(
            voice_id,
            name=name,
            language=language,
            description=description,
            labels=labels,
            request_options=request_options,
        )
        return _response.data

    def train(
        self,
        voice_id: str,
        *,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartPvcVoiceTrainingResponseModel:
        """
        Start PVC training process for a voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        model_id : typing.Optional[str]
            The model ID to use for the conversion.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartPvcVoiceTrainingResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.pvc.train(
            voice_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.train(voice_id, model_id=model_id, request_options=request_options)
        return _response.data

    @property
    def samples(self):
        if self._samples is None:
            from .samples.client import SamplesClient  # noqa: E402

            self._samples = SamplesClient(client_wrapper=self._client_wrapper)
        return self._samples

    @property
    def verification(self):
        if self._verification is None:
            from .verification.client import VerificationClient  # noqa: E402

            self._verification = VerificationClient(client_wrapper=self._client_wrapper)
        return self._verification


class AsyncPvcClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPvcClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._samples: typing.Optional[AsyncSamplesClient] = None
        self._verification: typing.Optional[AsyncVerificationClient] = None

    @property
    def with_raw_response(self) -> AsyncRawPvcClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPvcClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        name: str,
        language: str,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddVoiceResponseModel:
        """
        Creates a new PVC voice with metadata but no samples

        Parameters
        ----------
        name : str
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : str
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddVoiceResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.create(
                name="John Smith",
                language="en",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name, language=language, description=description, labels=labels, request_options=request_options
        )
        return _response.data

    async def update(
        self,
        voice_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddVoiceResponseModel:
        """
        Edit PVC voice metadata

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        name : typing.Optional[str]
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : typing.Optional[str]
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddVoiceResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.update(
                voice_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            voice_id,
            name=name,
            language=language,
            description=description,
            labels=labels,
            request_options=request_options,
        )
        return _response.data

    async def train(
        self,
        voice_id: str,
        *,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StartPvcVoiceTrainingResponseModel:
        """
        Start PVC training process for a voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        model_id : typing.Optional[str]
            The model ID to use for the conversion.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartPvcVoiceTrainingResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.pvc.train(
                voice_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.train(voice_id, model_id=model_id, request_options=request_options)
        return _response.data

    @property
    def samples(self):
        if self._samples is None:
            from .samples.client import AsyncSamplesClient  # noqa: E402

            self._samples = AsyncSamplesClient(client_wrapper=self._client_wrapper)
        return self._samples

    @property
    def verification(self):
        if self._verification is None:
            from .verification.client import AsyncVerificationClient  # noqa: E402

            self._verification = AsyncVerificationClient(client_wrapper=self._client_wrapper)
        return self._verification



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.conversation_initiation_client_data_request_input import ConversationInitiationClientDataRequestInput
from ...types.whats_app_outbound_call_response import WhatsAppOutboundCallResponse
from ...types.whats_app_outbound_message_response import WhatsAppOutboundMessageResponse
from .raw_client import AsyncRawWhatsappClient, RawWhatsappClient
from .types.body_send_an_outbound_message_via_whats_app_v_1_convai_whatsapp_outbound_message_post_template_params_item import (
    BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WhatsappClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWhatsappClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWhatsappClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWhatsappClient
        """
        return self._raw_client

    def outbound_call(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        whatsapp_call_permission_request_template_name: str,
        whatsapp_call_permission_request_template_language_code: str,
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhatsAppOutboundCallResponse:
        """
        Make an outbound call via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        whatsapp_call_permission_request_template_name : str

        whatsapp_call_permission_request_template_language_code : str

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhatsAppOutboundCallResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.whatsapp.outbound_call(
            whatsapp_phone_number_id="whatsapp_phone_number_id",
            whatsapp_user_id="whatsapp_user_id",
            whatsapp_call_permission_request_template_name="whatsapp_call_permission_request_template_name",
            whatsapp_call_permission_request_template_language_code="whatsapp_call_permission_request_template_language_code",
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.outbound_call(
            whatsapp_phone_number_id=whatsapp_phone_number_id,
            whatsapp_user_id=whatsapp_user_id,
            whatsapp_call_permission_request_template_name=whatsapp_call_permission_request_template_name,
            whatsapp_call_permission_request_template_language_code=whatsapp_call_permission_request_template_language_code,
            agent_id=agent_id,
            conversation_initiation_client_data=conversation_initiation_client_data,
            request_options=request_options,
        )
        return _response.data

    def outbound_message(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        template_name: str,
        template_language_code: str,
        template_params: typing.Sequence[
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
        ],
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhatsAppOutboundMessageResponse:
        """
        Send an outbound message via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        template_name : str

        template_language_code : str

        template_params : typing.Sequence[BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem]

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhatsAppOutboundMessageResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, WhatsAppTemplateTextParam
        from elevenlabs.conversational_ai.whatsapp import (
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem_Body,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.whatsapp.outbound_message(
            whatsapp_phone_number_id="whatsapp_phone_number_id",
            whatsapp_user_id="whatsapp_user_id",
            template_name="template_name",
            template_language_code="template_language_code",
            template_params=[
                BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem_Body(
                    parameters=[
                        WhatsAppTemplateTextParam(
                            text="text",
                        )
                    ],
                )
            ],
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.outbound_message(
            whatsapp_phone_number_id=whatsapp_phone_number_id,
            whatsapp_user_id=whatsapp_user_id,
            template_name=template_name,
            template_language_code=template_language_code,
            template_params=template_params,
            agent_id=agent_id,
            conversation_initiation_client_data=conversation_initiation_client_data,
            request_options=request_options,
        )
        return _response.data


class AsyncWhatsappClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWhatsappClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWhatsappClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWhatsappClient
        """
        return self._raw_client

    async def outbound_call(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        whatsapp_call_permission_request_template_name: str,
        whatsapp_call_permission_request_template_language_code: str,
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhatsAppOutboundCallResponse:
        """
        Make an outbound call via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        whatsapp_call_permission_request_template_name : str

        whatsapp_call_permission_request_template_language_code : str

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhatsAppOutboundCallResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.whatsapp.outbound_call(
                whatsapp_phone_number_id="whatsapp_phone_number_id",
                whatsapp_user_id="whatsapp_user_id",
                whatsapp_call_permission_request_template_name="whatsapp_call_permission_request_template_name",
                whatsapp_call_permission_request_template_language_code="whatsapp_call_permission_request_template_language_code",
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.outbound_call(
            whatsapp_phone_number_id=whatsapp_phone_number_id,
            whatsapp_user_id=whatsapp_user_id,
            whatsapp_call_permission_request_template_name=whatsapp_call_permission_request_template_name,
            whatsapp_call_permission_request_template_language_code=whatsapp_call_permission_request_template_language_code,
            agent_id=agent_id,
            conversation_initiation_client_data=conversation_initiation_client_data,
            request_options=request_options,
        )
        return _response.data

    async def outbound_message(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        template_name: str,
        template_language_code: str,
        template_params: typing.Sequence[
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
        ],
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WhatsAppOutboundMessageResponse:
        """
        Send an outbound message via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        template_name : str

        template_language_code : str

        template_params : typing.Sequence[BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem]

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WhatsAppOutboundMessageResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, WhatsAppTemplateTextParam
        from elevenlabs.conversational_ai.whatsapp import (
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem_Body,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.whatsapp.outbound_message(
                whatsapp_phone_number_id="whatsapp_phone_number_id",
                whatsapp_user_id="whatsapp_user_id",
                template_name="template_name",
                template_language_code="template_language_code",
                template_params=[
                    BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem_Body(
                        parameters=[
                            WhatsAppTemplateTextParam(
                                text="text",
                            )
                        ],
                    )
                ],
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.outbound_message(
            whatsapp_phone_number_id=whatsapp_phone_number_id,
            whatsapp_user_id=whatsapp_user_id,
            template_name=template_name,
            template_language_code=template_language_code,
            template_params=template_params,
            agent_id=agent_id,
            conversation_initiation_client_data=conversation_initiation_client_data,
            request_options=request_options,
        )
        return _response.data



import json
import hmac
import hashlib
import time
from typing import Any, Dict
from .types.bad_request_error_body import BadRequestErrorBody
from .errors import BadRequestError
from .webhooks.client import WebhooksClient as AutogeneratedWebhooksClient, AsyncWebhooksClient as AutogeneratedAsyncWebhooksClient


class WebhooksClient(AutogeneratedWebhooksClient):
    """
    A client to handle ElevenLabs webhook-related functionality
    Extends the autogenerated client to include custom webhook methods
    """

    def construct_event(self, rawBody: str, sig_header: str, secret: str) -> Dict:
        """
        Constructs a webhook event object from a payload and signature.
        Verifies the webhook signature to ensure the event came from ElevenLabs.

        Args:
            rawBody: The webhook request body. Must be the raw body, not a JSON object
            sig_header: The signature header from the request
            secret: Your webhook secret

        Returns:
            The verified webhook event

        Raises:
            BadRequestError: If the signature is invalid or missing
        """

        if not sig_header:
            raise BadRequestError(body=BadRequestErrorBody(error="Missing signature header"))

        if not secret:
            raise BadRequestError(body=BadRequestErrorBody(error="Webhook secret not configured"))

        headers = sig_header.split(',')
        timestamp = None
        signature = None

        for header in headers:
            if header.startswith('t='):
                timestamp = header[2:]
            elif header.startswith('v0='):
                signature = header

        if not timestamp or not signature:
            raise BadRequestError(body=BadRequestErrorBody(error="No signature hash found with expected scheme v0"))

        # Validate timestamp
        req_timestamp = int(timestamp) * 1000
        tolerance = int(time.time() * 1000) - 30 * 60 * 1000
        if req_timestamp < tolerance:
            raise BadRequestError(body=BadRequestErrorBody(error="Timestamp outside the tolerance zone"))

        # Validate hash
        message = f"{timestamp}.{rawBody}"

        digest = "v0=" + hmac.new(
            secret.encode('utf-8'),
            message.encode('utf-8'),
            hashlib.sha256
        ).hexdigest()

        if signature != digest:
            raise BadRequestError(
                body=BadRequestErrorBody(error="Signature hash does not match the expected signature hash for payload")
            )

        return json.loads(rawBody)


class AsyncWebhooksClient(AutogeneratedAsyncWebhooksClient):
    """
    Async version of WebhooksClient that extends the autogenerated async client
    """

    def construct_event(self, rawBody: str, sig_header: str, secret: str) -> Dict:
        """
        Constructs a webhook event object from a payload and signature.
        Verifies the webhook signature to ensure the event came from ElevenLabs.

        Args:
            rawBody: The webhook request body. Must be the raw body, not a JSON object
            sig_header: The signature header from the request
            secret: Your webhook secret

        Returns:
            The verified webhook event

        Raises:
            BadRequestError: If the signature is invalid or missing
        """

        if not sig_header:
            raise BadRequestError(body=BadRequestErrorBody(error="Missing signature header"))

        if not secret:
            raise BadRequestError(body=BadRequestErrorBody(error="Webhook secret not configured"))

        headers = sig_header.split(',')
        timestamp = None
        signature = None

        for header in headers:
            if header.startswith('t='):
                timestamp = header[2:]
            elif header.startswith('v0='):
                signature = header

        if not timestamp or not signature:
            raise BadRequestError(body=BadRequestErrorBody(error="No signature hash found with expected scheme v0"))

        # Validate timestamp
        req_timestamp = int(timestamp) * 1000
        tolerance = int(time.time() * 1000) - 30 * 60 * 1000
        if req_timestamp < tolerance:
            raise BadRequestError(body=BadRequestErrorBody(error="Timestamp outside the tolerance zone"))

        # Validate hash
        message = f"{timestamp}.{rawBody}"

        digest = "v0=" + hmac.new(
            secret.encode('utf-8'),
            message.encode('utf-8'),
            hashlib.sha256
        ).hexdigest()

        if signature != digest:
            raise BadRequestError(
                body=BadRequestErrorBody(error="Signature hash does not match the expected signature hash for payload")
            )

        return json.loads(rawBody)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class PlayDtmfToolConfig(UncheckedBaseModel):
    """
    Allows the agent to play DTMF tones during a phone call.

    This tool can be used to interact with automated phone systems, such as
    navigating phone menus, entering extensions, or inputting numeric codes.
    """

    use_out_of_band_dtmf: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Send DTMF tones as out-of-band RTP events (RFC 4733) instead of in-band audio. Only effective for SIP trunk imported numbers.
    """

    suppress_turn_after_dtmf: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, the agent will not generate further speech after playing DTMF tones. This prevents the agent's speech from interfering with IVR systems.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .config_entity_type import ConfigEntityType


class ConversationHistoryRedactionConfig(UncheckedBaseModel):
    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether conversation history redaction is enabled
    """

    entities: typing.Optional[typing.List[ConfigEntityType]] = pydantic.Field(default=None)
    """
    The entities to redact from the conversation transcript, audio and analysis. Use top-level types like 'name', 'email_address', or dot notation for specific subtypes like 'name.full_name'.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .open_ai_tool_choice_function import OpenAiToolChoiceFunction

OpenAiSessionConfigToolChoice = typing.Union[str, OpenAiToolChoiceFunction]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.dubbing_transcript_response_model import DubbingTranscriptResponseModel

TranscriptGetTranscriptForDubResponse = typing.Union[DubbingTranscriptResponseModel, str]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .dynamic_variable_assignment import DynamicVariableAssignment
from .mcp_tool_config_override_input_input_overrides_value import McpToolConfigOverrideInputInputOverridesValue
from .pre_tool_speech_mode import PreToolSpeechMode
from .tool_call_sound_behavior import ToolCallSoundBehavior
from .tool_call_sound_type import ToolCallSoundType
from .tool_execution_mode import ToolExecutionMode
from .tool_response_mock_config_input import ToolResponseMockConfigInput


class McpToolConfigOverrideInput(UncheckedBaseModel):
    tool_name: str = pydantic.Field()
    """
    The name of the MCP tool
    """

    force_pre_tool_speech: typing.Optional[bool] = pydantic.Field(default=None)
    """
    DEPRECATED: use `pre_tool_speech` instead. If set, overrides the server's force_pre_tool_speech setting for this tool.
    """

    pre_tool_speech: typing.Optional[PreToolSpeechMode] = pydantic.Field(default=None)
    """
    If set, overrides the server's pre_tool_speech setting for this tool.
    """

    disable_interruptions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If set, overrides the server's disable_interruptions setting for this tool
    """

    tool_call_sound: typing.Optional[ToolCallSoundType] = pydantic.Field(default=None)
    """
    If set, overrides the server's tool_call_sound setting for this tool
    """

    tool_call_sound_behavior: typing.Optional[ToolCallSoundBehavior] = pydantic.Field(default=None)
    """
    If set, overrides the server's tool_call_sound_behavior setting for this tool
    """

    execution_mode: typing.Optional[ToolExecutionMode] = pydantic.Field(default=None)
    """
    If set, overrides the server's execution_mode setting for this tool
    """

    response_timeout_secs: typing.Optional[int] = pydantic.Field(default=None)
    """
    If set, overrides the server's response timeout for this MCP tool (seconds).
    """

    assignments: typing.Optional[typing.List[DynamicVariableAssignment]] = pydantic.Field(default=None)
    """
    Dynamic variable assignments for this MCP tool
    """

    input_overrides: typing.Optional[
        typing.Dict[str, typing.Optional[McpToolConfigOverrideInputInputOverridesValue]]
    ] = pydantic.Field(default=None)
    """
    Mapping of json path to input override configuration
    """

    response_mocks: typing.Optional[typing.List[ToolResponseMockConfigInput]] = pydantic.Field(default=None)
    """
    Mock responses with optional parameter conditions. Evaluated top-to-bottom; first match wins.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class AgentVersionParents(UncheckedBaseModel):
    in_branch_parent_id: typing.Optional[str] = None
    out_of_branch_parent_id: typing.Optional[str] = None
    merged_into_branch_id: typing.Optional[str] = None
    merged_from_branch_id: typing.Optional[str] = None
    merged_from_version_id: typing.Optional[str] = None
    rebased_from_version_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .cue_options_request import CueOptionsRequest
from .media_id import MediaId


class SubtitleOrderItemRequest(UncheckedBaseModel):
    media_ids: typing.List[MediaId] = pydantic.Field()
    """
    The IDs of the uploaded media files to generate subtitles for.
    """

    source_language: str = pydantic.Field()
    """
    The language code of the source media (e.g. 'en', 'es').
    """

    destination_languages: typing.List[str] = pydantic.Field()
    """
    List of target language codes. Subtitles will be generated for each media file in each destination language.
    """

    cue_options: typing.Optional[CueOptionsRequest] = pydantic.Field(default=None)
    """
    Formatting options for subtitle cues such as duration, line count, and character limits.
    """

    sdh: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether subtitles should use SDH format, which includes descriptions for deaf and hard-of-hearing viewers.
    """

    instructions: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional free-text instructions for the subtitling team.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from .types.body_generate_composition_plan_v_1_music_plan_post_model_id import (
    BodyGenerateCompositionPlanV1MusicPlanPostModelId,
)
from .types.body_generate_composition_plan_v_1_music_plan_post_source_composition_plan import (
    BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan,
)
from .types.composition_plan_create_response import CompositionPlanCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCompositionPlanClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        prompt: str,
        music_length_ms: typing.Optional[int] = OMIT,
        source_composition_plan: typing.Optional[
            BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan
        ] = OMIT,
        model_id: typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostModelId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CompositionPlanCreateResponse]:
        """
        Create a composition plan for music generation. Usage of this endpoint does not cost any credits but is subject to rate limiting depending on your tier.

        Parameters
        ----------
        prompt : str
            A simple text prompt to compose a plan from.

        music_length_ms : typing.Optional[int]
            The length of the composition plan to generate in milliseconds. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        source_composition_plan : typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan]
            An optional composition plan to use as a source for the new composition plan.

        model_id : typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostModelId]
            The model to use for the generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CompositionPlanCreateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/music/plan",
            method="POST",
            json={
                "prompt": prompt,
                "music_length_ms": music_length_ms,
                "source_composition_plan": convert_and_respect_annotation_metadata(
                    object_=source_composition_plan,
                    annotation=BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan,
                    direction="write",
                ),
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CompositionPlanCreateResponse,
                    construct_type(
                        type_=CompositionPlanCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCompositionPlanClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        prompt: str,
        music_length_ms: typing.Optional[int] = OMIT,
        source_composition_plan: typing.Optional[
            BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan
        ] = OMIT,
        model_id: typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostModelId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CompositionPlanCreateResponse]:
        """
        Create a composition plan for music generation. Usage of this endpoint does not cost any credits but is subject to rate limiting depending on your tier.

        Parameters
        ----------
        prompt : str
            A simple text prompt to compose a plan from.

        music_length_ms : typing.Optional[int]
            The length of the composition plan to generate in milliseconds. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        source_composition_plan : typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan]
            An optional composition plan to use as a source for the new composition plan.

        model_id : typing.Optional[BodyGenerateCompositionPlanV1MusicPlanPostModelId]
            The model to use for the generation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CompositionPlanCreateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/music/plan",
            method="POST",
            json={
                "prompt": prompt,
                "music_length_ms": music_length_ms,
                "source_composition_plan": convert_and_respect_annotation_metadata(
                    object_=source_composition_plan,
                    annotation=BodyGenerateCompositionPlanV1MusicPlanPostSourceCompositionPlan,
                    direction="write",
                ),
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CompositionPlanCreateResponse,
                    construct_type(
                        type_=CompositionPlanCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .position_output import PositionOutput
from .transfer_type_enum import TransferTypeEnum
from .workflow_phone_number_node_model_output_custom_sip_headers_item import (
    WorkflowPhoneNumberNodeModelOutputCustomSipHeadersItem,
)
from .workflow_phone_number_node_model_output_post_dial_digits import WorkflowPhoneNumberNodeModelOutputPostDialDigits
from .workflow_phone_number_node_model_output_transfer_destination import (
    WorkflowPhoneNumberNodeModelOutputTransferDestination,
)


class WorkflowPhoneNumberNodeModelOutput(UncheckedBaseModel):
    custom_sip_headers: typing.List[WorkflowPhoneNumberNodeModelOutputCustomSipHeadersItem] = pydantic.Field()
    """
    Custom SIP headers to include when transferring the call. Each header can be either a static value or a dynamic variable reference.
    """

    transfer_destination: WorkflowPhoneNumberNodeModelOutputTransferDestination
    transfer_type: TransferTypeEnum
    post_dial_digits: typing.Optional[WorkflowPhoneNumberNodeModelOutputPostDialDigits] = pydantic.Field(default=None)
    """
    DTMF digits to send after call connects (e.g., 'ww1234' for extension). Can be either a static value or a dynamic variable reference. Use 'w' for 0.5s pause. Only supported for Twilio transfers.
    """

    position: PositionOutput = pydantic.Field()
    """
    Position of the node in the workflow.
    """

    edge_order: typing.List[str] = pydantic.Field()
    """
    The ids of outgoing edges in the order they should be evaluated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""SpeechEngineSession — WebSocket session for Speech Engine conversations."""

import asyncio
import json
import logging
import typing

from .types import ConversationMessage, WebSocketLike, wrap_websocket

logger = logging.getLogger("elevenlabs.speech_engine")


def _make_log(
    debug: bool,
) -> typing.Callable[..., None]:
    """Return a per-instance log function, mirroring the JS SDK pattern."""
    if debug:
        def _log(msg: str, *args: typing.Any) -> None:
            print("[SpeechEngine]", msg % args if args else msg)
        return _log
    return lambda *_args, **_kw: None

Callback = typing.Callable[..., typing.Any]


# ---------------------------------------------------------------------------
# LLM stream text extraction
# ---------------------------------------------------------------------------


def _get(obj: typing.Any, key: str) -> typing.Any:
    """Attribute-or-key access helper for LLM stream chunks."""
    if isinstance(obj, dict):
        return obj.get(key)
    return getattr(obj, key, None)


def _extract_text(chunk: typing.Any) -> typing.Optional[str]:
    """Extract text content from an LLM stream chunk.

    Handles plain strings and common LLM streaming formats:

    - OpenAI Responses API  (``response.output_text.delta``)
    - OpenAI Chat Completions API (``choices[0].delta.content``)
    - Anthropic Messages API (``content_block_delta`` with ``text_delta``)
    - Google Gemini API (``candidates[0].content.parts[0].text``)
    """
    if isinstance(chunk, str):
        return chunk
    if chunk is None or isinstance(chunk, (int, float, bool)):
        return None

    # OpenAI Responses API
    if _get(chunk, "type") == "response.output_text.delta":
        delta = _get(chunk, "delta")
        if isinstance(delta, str):
            return delta

    # OpenAI Chat Completions API
    choices = _get(chunk, "choices")
    if isinstance(choices, (list, tuple)) and len(choices) > 0:
        delta = _get(choices[0], "delta")
        if delta is not None:
            content = _get(delta, "content")
            if isinstance(content, str):
                return content

    # Anthropic Messages API
    if _get(chunk, "type") == "content_block_delta":
        delta = _get(chunk, "delta")
        if delta is not None:
            if _get(delta, "type") == "text_delta":
                text = _get(delta, "text")
                if isinstance(text, str):
                    return text

    # Google Gemini API
    candidates = _get(chunk, "candidates")
    if isinstance(candidates, (list, tuple)) and len(candidates) > 0:
        content = _get(candidates[0], "content")
        if content is not None:
            parts = _get(content, "parts")
            if isinstance(parts, (list, tuple)) and len(parts) > 0:
                text = _get(parts[0], "text")
                if isinstance(text, str):
                    return text

    return None


# ---------------------------------------------------------------------------
# Handler wiring (kwargs -> event emitter)
# ---------------------------------------------------------------------------


def _wire_handlers(
    session: "SpeechEngineSession",
    handlers: typing.Dict[str, typing.Any],
) -> None:
    """Wire keyword-argument handlers onto *session* events."""
    on_init = handlers.get("on_init")
    on_transcript = handlers.get("on_transcript")
    on_close = handlers.get("on_close")
    on_disconnect = handlers.get("on_disconnect")
    on_error = handlers.get("on_error")

    if on_init:
        async def _init_handler(conversation_id: str) -> None:
            result = on_init(conversation_id, session)
            if asyncio.iscoroutine(result):
                await result
        session.on("init", _init_handler)

    if on_transcript:
        async def _transcript_handler(
            transcript: typing.List[ConversationMessage],
        ) -> None:
            result = on_transcript(transcript, session)
            if asyncio.iscoroutine(result):
                await result
        session.on("user_transcript", _transcript_handler)

    if on_close:
        async def _close_handler() -> None:
            result = on_close(session)
            if asyncio.iscoroutine(result):
                await result
        session.on("close", _close_handler)

    if on_disconnect:
        async def _disconnect_handler() -> None:
            result = on_disconnect(session)
            if asyncio.iscoroutine(result):
                await result
        session.on("disconnected", _disconnect_handler)

    if on_error:
        async def _error_handler(err: Exception) -> None:
            result = on_error(err, session)
            if asyncio.iscoroutine(result):
                await result
        session.on("error", _error_handler)


# ---------------------------------------------------------------------------
# SpeechEngineSession
# ---------------------------------------------------------------------------


class SpeechEngineSession:
    """Wraps a WebSocket connection from the ElevenLabs Speech Engine API.

    Each connection represents one conversation.  The session emits events
    for transcripts and lifecycle changes, and provides methods to send LLM
    responses back.  When a new transcript arrives the previous transcript's
    handler task is cancelled automatically, interrupting any in-flight LLM
    call.

    Example::

        session = SpeechEngineSession(ws, debug=True)

        async def handle(transcript):
            stream = await openai.responses.create(...)
            await session.send_response(stream)

        session.on("user_transcript", handle)
        await session.run()
    """

    def __init__(
        self,
        ws: typing.Any,
        *,
        debug: bool = False,
    ) -> None:
        self._ws = wrap_websocket(ws)
        self._conversation_id = None  # type: typing.Optional[str]
        self._current_task = None  # type: typing.Optional[asyncio.Task]  # type: ignore[type-arg]
        self._current_event_id = None  # type: typing.Optional[int]
        self._in_transcript_handler = False
        self._closed = False
        self._event_handlers = {}  # type: typing.Dict[str, typing.List[Callback]]
        self._once_handlers = {}  # type: typing.Dict[str, typing.List[Callback]]
        self._log = _make_log(debug)

    # ------------------------------------------------------------------
    # Event emitter interface
    # ------------------------------------------------------------------

    def on(self, event: str, handler: Callback) -> "SpeechEngineSession":
        """Register *handler* for *event*."""
        self._event_handlers.setdefault(event, []).append(handler)
        return self

    def off(self, event: str, handler: Callback) -> "SpeechEngineSession":
        """Remove *handler* from *event*."""
        handlers = self._event_handlers.get(event, [])
        try:
            handlers.remove(handler)
        except ValueError:
            pass
        return self

    def once(self, event: str, handler: Callback) -> "SpeechEngineSession":
        """Register *handler* for *event*, removed after the first call."""
        self._once_handlers.setdefault(event, []).append(handler)
        return self

    # ------------------------------------------------------------------
    # Properties
    # ------------------------------------------------------------------

    @property
    def conversation_id(self) -> typing.Optional[str]:
        """The conversation ID assigned by the Speech Engine API.

        Available after the ``init`` event fires.
        """
        return self._conversation_id

    @property
    def is_open(self) -> bool:
        """Whether the session is still open."""
        return not self._closed

    # ------------------------------------------------------------------
    # Main message loop
    # ------------------------------------------------------------------

    async def run(self) -> None:
        """Run the receive loop until the WebSocket closes.

        This is the main entry point after constructing a session.  It
        processes incoming messages and dispatches events to registered
        handlers.
        """
        try:
            while not self._closed:
                try:
                    raw = await self._ws.recv()
                except asyncio.CancelledError:
                    raise
                except Exception:
                    self._log("WebSocket connection lost")
                    break

                try:
                    if isinstance(raw, bytes):
                        raw = raw.decode("utf-8")
                    msg = json.loads(raw)
                except (ValueError, TypeError, UnicodeDecodeError) as e:
                    await self._emit("error", e)
                    continue

                if not isinstance(msg, dict):
                    await self._emit(
                        "error",
                        ValueError(f"expected JSON object, got {type(msg).__name__}"),
                    )
                    continue

                await self._handle_message(msg)
        except asyncio.CancelledError:
            raise
        finally:
            if not self._closed:
                self._closed = True
                await self._cancel_current_and_wait()
                await self._emit("disconnected")

    # ------------------------------------------------------------------
    # Sending responses
    # ------------------------------------------------------------------

    async def send_response(
        self,
        response: typing.Any,
    ) -> None:
        """Send an LLM response back for TTS synthesis.

        Accepts:

        * A plain **string** — sent as a single agent response.
        * An **async iterator** yielding strings or LLM stream event objects
          (OpenAI, Anthropic, Gemini formats are auto-detected).

        This method is a coroutine so the caller can ``await`` it to know
        when the full response has been sent.
        """
        if self._closed:
            raise RuntimeError("Cannot send response: session is closed")

        if not self._in_transcript_handler:
            logger.warning(
                "send_response() called outside of an on_transcript handler. "
                "Responses can only be sent in reply to a user transcript. "
                "To have the agent speak first, set a first message in your "
                "Speech Engine conversation config on the client."
            )
            return

        event_id = self._current_event_id

        if isinstance(response, str):
            self._log(
                'sending string response: "%s", event_id=%s',
                response,
                event_id,
            )
            await self._send_agent_response(response, False, event_id)
            await self._send_agent_response("", True, event_id)
        else:
            self._log(
                "starting streamed response, event_id=%s",
                event_id,
            )
            await self._stream_response(response, event_id)

    def close(self) -> None:
        """Close the session and the underlying WebSocket connection."""
        if self._closed:
            return
        self._closed = True
        self._cancel_current()
        try:
            asyncio.ensure_future(self._ws.close())
        except RuntimeError:
            # No running event loop — best-effort.
            pass

    # ------------------------------------------------------------------
    # Internals
    # ------------------------------------------------------------------

    async def _handle_message(self, msg: typing.Dict[str, typing.Any]) -> None:
        msg_type = msg.get("type")

        if msg_type == "init":
            self._conversation_id = msg.get("conversation_id")
            self._log(
                "session initialized, conversation_id=%s",
                self._conversation_id,
            )
            await self._emit("init", self._conversation_id)

        elif msg_type == "user_transcript":
            incoming_event_id = msg.get("event_id")

            if (
                incoming_event_id is not None
                and incoming_event_id == self._current_event_id
                and self._current_task is not None
                and not self._current_task.done()
            ):
                self._log(
                    "skipping duplicate transcript, event_id=%s",
                    incoming_event_id,
                )
                return

            was_active = (
                self._current_task is not None
                and not self._current_task.done()
            )
            await self._cancel_current_and_wait()
            if was_active:
                self._log(
                    "interrupted: cancelling previous response "
                    "(event_id=%s) for new transcript (event_id=%s)",
                    self._current_event_id,
                    incoming_event_id,
                )

            self._current_event_id = incoming_event_id
            transcript_data = msg.get("user_transcript") or []

            try:
                transcript = [
                    ConversationMessage(role=m["role"], content=m["content"])
                    for m in transcript_data
                ]
            except (KeyError, TypeError) as e:
                await self._emit("error", e)
                return

            self._log(
                "received transcript, event_id=%s, messages=%d",
                self._current_event_id,
                len(transcript),
            )

            handlers = list(
                self._event_handlers.get("user_transcript", [])
            )
            once_handlers = self._once_handlers.pop("user_transcript", [])
            all_handlers = handlers + once_handlers

            if all_handlers:
                self._current_task = asyncio.create_task(
                    self._run_transcript_handlers(all_handlers, transcript)
                )
                # Yield so the handler task can start before the next
                # message is read.  This mirrors the JS behaviour where
                # emitter.emit() invokes listeners synchronously.
                await asyncio.sleep(0)

        elif msg_type == "ping":
            await self._send({"type": "pong"})

        elif msg_type == "close":
            self._closed = True
            self._in_transcript_handler = False
            await self._cancel_current_and_wait()
            await self._emit("close")

        elif msg_type == "error":
            await self._emit("error", Exception(msg.get("message", "")))

        # Unknown types are silently ignored for forward compatibility.

    async def _run_transcript_handlers(
        self,
        handlers: typing.List[Callback],
        transcript: typing.List[ConversationMessage],
    ) -> None:
        self._in_transcript_handler = True
        try:
            for handler in handlers:
                result = handler(transcript)
                if asyncio.iscoroutine(result):
                    await result
        except asyncio.CancelledError:
            raise
        except Exception as e:
            await self._emit("error", e)
        finally:
            self._in_transcript_handler = False

    async def _stream_response(
        self, stream: typing.Any, event_id: typing.Optional[int] = None
    ) -> None:
        chunks = 0
        try:
            async for chunk in stream:
                if self._closed:
                    self._log(
                        "stream stopped: session closed after %d chunks, "
                        "event_id=%s",
                        chunks,
                        event_id,
                    )
                    return
                text = _extract_text(chunk)
                if text:
                    chunks += 1
                    await self._send_agent_response(text, False, event_id)
            if not self._closed:
                self._log(
                    "stream complete: %d chunks sent, event_id=%s",
                    chunks,
                    event_id,
                )
                await self._send_agent_response("", True, event_id)
        except asyncio.CancelledError:
            raise
        except Exception as e:
            await self._emit("error", e)
            if not self._closed:
                await self._send_agent_response("", True, event_id)

    async def _send_agent_response(
        self,
        content: str,
        is_final: bool,
        event_id: typing.Optional[int] = None,
    ) -> None:
        if event_id is None:
            event_id = self._current_event_id
        await self._send(
            {
                "type": "agent_response",
                "content": content,
                "event_id": event_id,
                "is_final": is_final,
            }
        )

    async def _send(self, msg: typing.Dict[str, typing.Any]) -> None:
        if self._closed:
            return
        try:
            await self._ws.send(json.dumps(msg))
        except asyncio.CancelledError:
            raise
        except Exception:
            # Send failed — the recv loop will detect the closed connection.
            pass

    async def _emit(self, event: str, *args: typing.Any) -> None:
        handlers = list(self._event_handlers.get(event, []))
        once_handlers = self._once_handlers.pop(event, [])
        all_handlers = handlers + once_handlers

        for handler in all_handlers:
            try:
                result = handler(*args)
                if asyncio.iscoroutine(result):
                    await result
            except asyncio.CancelledError:
                raise
            except Exception as e:
                if event != "error":
                    await self._emit("error", e)
                else:
                    logger.exception("unhandled error in error handler: %s", e)

    async def _cancel_current_and_wait(self) -> None:
        """Cancel the current handler task and wait for cleanup."""
        task = self._current_task
        self._current_task = None
        if task is not None and not task.done():
            task.cancel()
            try:
                await task
            except (asyncio.CancelledError, Exception):
                pass

    def _cancel_current(self) -> None:
        """Cancel the current handler task (fire-and-forget)."""
        if self._current_task is not None and not self._current_task.done():
            self._current_task.cancel()
        self._current_task = None



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .auth_connection_dependencies import AuthConnectionDependencies


class OAuth2ClientCredsResponse(UncheckedBaseModel):
    """
    Response model for oauth2 client creds
    """

    name: str
    provider: str
    client_id: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    extra_params: typing.Optional[typing.Dict[str, str]] = None
    basic_auth_in_header: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If True, send client credentials in Authorization header as Basic Auth instead of request body
    """

    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None
    custom_headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Custom headers configured for OAuth2 token requests
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class DetectedEntity(UncheckedBaseModel):
    text: str = pydantic.Field()
    """
    The text that was identified as an entity.
    """

    entity_type: str = pydantic.Field()
    """
    The type of entity detected (e.g., 'credit_card', 'email_address', 'person_name').
    """

    start_char: int = pydantic.Field()
    """
    Start character position in the transcript text.
    """

    end_char: int = pydantic.Field()
    """
    End character position in the transcript text.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .position_output import PositionOutput


class WorkflowEndNodeModelOutput(UncheckedBaseModel):
    position: PositionOutput = pydantic.Field()
    """
    Position of the node in the workflow.
    """

    edge_order: typing.List[str] = pydantic.Field()
    """
    The ids of outgoing edges in the order they should be evaluated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_history_transcript_tool_call_webhook_details import (
    ConversationHistoryTranscriptToolCallWebhookDetails,
)


class ConversationHistoryTranscriptToolCallApiIntegrationWebhookDetailsInput(UncheckedBaseModel):
    integration_id: typing.Optional[str] = None
    credential_id: typing.Optional[str] = None
    integration_connection_id: typing.Optional[str] = None
    webhook_details: ConversationHistoryTranscriptToolCallWebhookDetails

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawProductionsClient, RawProductionsClient

if typing.TYPE_CHECKING:
    from .orders.client import AsyncOrdersClient, OrdersClient


class ProductionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProductionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._orders: typing.Optional[OrdersClient] = None

    @property
    def with_raw_response(self) -> RawProductionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProductionsClient
        """
        return self._raw_client

    @property
    def orders(self):
        if self._orders is None:
            from .orders.client import OrdersClient  # noqa: E402

            self._orders = OrdersClient(client_wrapper=self._client_wrapper)
        return self._orders


class AsyncProductionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProductionsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._orders: typing.Optional[AsyncOrdersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawProductionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProductionsClient
        """
        return self._raw_client

    @property
    def orders(self):
        if self._orders is None:
            from .orders.client import AsyncOrdersClient  # noqa: E402

            self._orders = AsyncOrdersClient(client_wrapper=self._client_wrapper)
        return self._orders



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .pydantic_pronunciation_dictionary_version_locator import PydanticPronunciationDictionaryVersionLocator
from .suggested_audio_tag import SuggestedAudioTag
from .supported_voice import SupportedVoice
from .text_normalisation_type import TextNormalisationType
from .tts_conversational_model import TtsConversationalModel
from .tts_optimize_streaming_latency import TtsOptimizeStreamingLatency
from .tts_output_format import TtsOutputFormat


class TtsConversationalConfigWorkflowOverrideInput(UncheckedBaseModel):
    model_id: typing.Optional[TtsConversationalModel] = pydantic.Field(default=None)
    """
    The model to use for TTS
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The voice ID to use for TTS
    """

    supported_voices: typing.Optional[typing.List[SupportedVoice]] = pydantic.Field(default=None)
    """
    Additional supported voices for the agent
    """

    expressive_mode: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, applies expressive audio tags prompt. Automatically disabled for non-v3 models.
    """

    suggested_audio_tags: typing.Optional[typing.List[SuggestedAudioTag]] = pydantic.Field(default=None)
    """
    Suggested audio tags to boost expressive speech (for eleven_v3 and eleven_v3_conversational models). The agent can still use other tags not listed here.
    """

    agent_output_audio_format: typing.Optional[TtsOutputFormat] = pydantic.Field(default=None)
    """
    The audio format to use for TTS
    """

    optimize_streaming_latency: typing.Optional[TtsOptimizeStreamingLatency] = pydantic.Field(default=None)
    """
    The optimization for streaming latency
    """

    stability: typing.Optional[float] = pydantic.Field(default=None)
    """
    The stability of generated speech
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    The speed of generated speech
    """

    similarity_boost: typing.Optional[float] = pydantic.Field(default=None)
    """
    The similarity boost for generated speech
    """

    text_normalisation_type: typing.Optional[TextNormalisationType] = pydantic.Field(default=None)
    """
    Method for converting numbers to words before converting text to speech. If set to SYSTEM_PROMPT, the system prompt will be updated to include normalization instructions. If set to ELEVENLABS, the text will be normalized after generation, incurring slight additional latency.
    """

    pronunciation_dictionary_locators: typing.Optional[typing.List[PydanticPronunciationDictionaryVersionLocator]] = (
        pydantic.Field(default=None)
    )
    """
    The pronunciation dictionary locators
    """

    enable_phoneme_tags: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Opt-in to SSML phoneme tag handling for V3 models. When enabled, phoneme tags (inline and from pronunciation dictionaries) are parsed into inline IPA before being sent to the model.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .... import core
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.add_knowledge_base_response_model import AddKnowledgeBaseResponseModel
from ....types.get_knowledge_base_dependent_agents_response_model import GetKnowledgeBaseDependentAgentsResponseModel
from ....types.knowledge_base_dependent_type import KnowledgeBaseDependentType
from ....types.knowledge_base_source_file_url_response_model import KnowledgeBaseSourceFileUrlResponseModel
from .raw_client import AsyncRawDocumentsClient, RawDocumentsClient
from .types.documents_get_response import DocumentsGetResponse
from .types.documents_update_response import DocumentsUpdateResponse

if typing.TYPE_CHECKING:
    from .chunk.client import AsyncChunkClient, ChunkClient
    from .chunks.client import AsyncChunksClient, ChunksClient
    from .summaries.client import AsyncSummariesClient, SummariesClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDocumentsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._summaries: typing.Optional[SummariesClient] = None
        self._chunk: typing.Optional[ChunkClient] = None
        self._chunks: typing.Optional[ChunksClient] = None

    @property
    def with_raw_response(self) -> RawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDocumentsClient
        """
        return self._raw_client

    def create_from_url(
        self,
        *,
        url: str,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        enable_auto_sync: typing.Optional[bool] = OMIT,
        auto_remove: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document generated by scraping the given webpage.

        Parameters
        ----------
        url : str
            URL to a page of documentation that the agent will have access to in order to interact with users.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        enable_auto_sync : typing.Optional[bool]
            Whether to enable auto-sync for this URL document.

        auto_remove : typing.Optional[bool]
            Whether to automatically remove the document if the URL becomes unavailable. Only applicable when auto-sync is enabled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.create_from_url(
            url="url",
        )
        """
        _response = self._raw_client.create_from_url(
            url=url,
            name=name,
            parent_folder_id=parent_folder_id,
            enable_auto_sync=enable_auto_sync,
            auto_remove=auto_remove,
            request_options=request_options,
        )
        return _response.data

    def create_from_file(
        self,
        *,
        file: core.File,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document generated form the uploaded file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.create_from_file()
        """
        _response = self._raw_client.create_from_file(
            file=file, name=name, parent_folder_id=parent_folder_id, request_options=request_options
        )
        return _response.data

    def create_from_text(
        self,
        *,
        text: str,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document containing the provided text.

        Parameters
        ----------
        text : str
            Text content to be added to the knowledge base.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.create_from_text(
            text="text",
        )
        """
        _response = self._raw_client.create_from_text(
            text=text, name=name, parent_folder_id=parent_folder_id, request_options=request_options
        )
        return _response.data

    def create_folder(
        self,
        *,
        name: str,
        parent_folder_id: typing.Optional[str] = OMIT,
        enable_auto_sync: typing.Optional[bool] = OMIT,
        auto_remove: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a folder used for grouping documents together.

        Parameters
        ----------
        name : str
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        enable_auto_sync : typing.Optional[bool]
            Whether to enable auto-sync for this URL document.

        auto_remove : typing.Optional[bool]
            Whether to automatically remove the document if the URL becomes unavailable. Only applicable when auto-sync is enabled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.create_folder(
            name="name",
        )
        """
        _response = self._raw_client.create_folder(
            name=name,
            parent_folder_id=parent_folder_id,
            enable_auto_sync=enable_auto_sync,
            auto_remove=auto_remove,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        documentation_id: str,
        *,
        agent_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsGetResponse:
        """
        Get details about a specific documentation making up the agent's knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        agent_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.get(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.get(documentation_id, agent_id=agent_id, request_options=request_options)
        return _response.data

    def delete(
        self,
        documentation_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Delete a document or folder from the knowledge base.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        force : typing.Optional[bool]
            If set to true, the document or folder will be deleted regardless of whether it is used by any agents and it will be removed from the dependent agents. For non-empty folders, this will also delete all child documents and folders.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.delete(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
            force=True,
        )
        """
        _response = self._raw_client.delete(documentation_id, force=force, request_options=request_options)
        return _response.data

    def update(
        self,
        documentation_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsUpdateResponse:
        """
        Update the name and/or content of a document.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        content : typing.Optional[str]
            Updated content for the document. Only supported for text documents, URL documents with auto-sync disabled, and file documents.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsUpdateResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.update(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.update(
            documentation_id, name=name, content=content, request_options=request_options
        )
        return _response.data

    def get_agents(
        self,
        documentation_id: str,
        *,
        dependent_type: typing.Optional[KnowledgeBaseDependentType] = None,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseDependentAgentsResponseModel:
        """
        Get a list of agents depending on this knowledge base document

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        dependent_type : typing.Optional[KnowledgeBaseDependentType]
            Type of dependent agents to return.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseDependentAgentsResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.get_agents(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
            dependent_type="direct",
            page_size=1,
            cursor="cursor",
        )
        """
        _response = self._raw_client.get_agents(
            documentation_id,
            dependent_type=dependent_type,
            page_size=page_size,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    def get_content(self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> str:
        """
        Get the entire content of a document from the knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Streaming document content

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.get_content(
            documentation_id="documentation_id",
        )
        """
        _response = self._raw_client.get_content(documentation_id, request_options=request_options)
        return _response.data

    def get_source_file_url(
        self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> KnowledgeBaseSourceFileUrlResponseModel:
        """
        Get a signed URL to download the original source file of a file-type document from the knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseSourceFileUrlResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.get_source_file_url(
            documentation_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.get_source_file_url(documentation_id, request_options=request_options)
        return _response.data

    def move(
        self,
        document_id: str,
        *,
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Moves the entity from one folder to another.

        Parameters
        ----------
        document_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.move(
            document_id="21m00Tcm4TlvDq8ikWAM",
        )
        """
        _response = self._raw_client.move(document_id, move_to=move_to, request_options=request_options)
        return _response.data

    def bulk_move(
        self,
        *,
        document_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Moves multiple entities from one folder to another.

        Parameters
        ----------
        document_ids : typing.Sequence[str]
            The ids of documents or folders from the knowledge base.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.documents.bulk_move(
            document_ids=["21m00Tcm4TlvDq8ikWAM", "31m00Tcm4TlvDq8ikWBM"],
        )
        """
        _response = self._raw_client.bulk_move(
            document_ids=document_ids, move_to=move_to, request_options=request_options
        )
        return _response.data

    @property
    def summaries(self):
        if self._summaries is None:
            from .summaries.client import SummariesClient  # noqa: E402

            self._summaries = SummariesClient(client_wrapper=self._client_wrapper)
        return self._summaries

    @property
    def chunk(self):
        if self._chunk is None:
            from .chunk.client import ChunkClient  # noqa: E402

            self._chunk = ChunkClient(client_wrapper=self._client_wrapper)
        return self._chunk

    @property
    def chunks(self):
        if self._chunks is None:
            from .chunks.client import ChunksClient  # noqa: E402

            self._chunks = ChunksClient(client_wrapper=self._client_wrapper)
        return self._chunks


class AsyncDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDocumentsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._summaries: typing.Optional[AsyncSummariesClient] = None
        self._chunk: typing.Optional[AsyncChunkClient] = None
        self._chunks: typing.Optional[AsyncChunksClient] = None

    @property
    def with_raw_response(self) -> AsyncRawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDocumentsClient
        """
        return self._raw_client

    async def create_from_url(
        self,
        *,
        url: str,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        enable_auto_sync: typing.Optional[bool] = OMIT,
        auto_remove: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document generated by scraping the given webpage.

        Parameters
        ----------
        url : str
            URL to a page of documentation that the agent will have access to in order to interact with users.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        enable_auto_sync : typing.Optional[bool]
            Whether to enable auto-sync for this URL document.

        auto_remove : typing.Optional[bool]
            Whether to automatically remove the document if the URL becomes unavailable. Only applicable when auto-sync is enabled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.create_from_url(
                url="url",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_url(
            url=url,
            name=name,
            parent_folder_id=parent_folder_id,
            enable_auto_sync=enable_auto_sync,
            auto_remove=auto_remove,
            request_options=request_options,
        )
        return _response.data

    async def create_from_file(
        self,
        *,
        file: core.File,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document generated form the uploaded file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.create_from_file()


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_file(
            file=file, name=name, parent_folder_id=parent_folder_id, request_options=request_options
        )
        return _response.data

    async def create_from_text(
        self,
        *,
        text: str,
        name: typing.Optional[str] = OMIT,
        parent_folder_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a knowledge base document containing the provided text.

        Parameters
        ----------
        text : str
            Text content to be added to the knowledge base.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.create_from_text(
                text="text",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_from_text(
            text=text, name=name, parent_folder_id=parent_folder_id, request_options=request_options
        )
        return _response.data

    async def create_folder(
        self,
        *,
        name: str,
        parent_folder_id: typing.Optional[str] = OMIT,
        enable_auto_sync: typing.Optional[bool] = OMIT,
        auto_remove: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddKnowledgeBaseResponseModel:
        """
        Create a folder used for grouping documents together.

        Parameters
        ----------
        name : str
            A custom, human-readable name for the document.

        parent_folder_id : typing.Optional[str]
            If set, the created document or folder will be placed inside the given folder.

        enable_auto_sync : typing.Optional[bool]
            Whether to enable auto-sync for this URL document.

        auto_remove : typing.Optional[bool]
            Whether to automatically remove the document if the URL becomes unavailable. Only applicable when auto-sync is enabled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddKnowledgeBaseResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.create_folder(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_folder(
            name=name,
            parent_folder_id=parent_folder_id,
            enable_auto_sync=enable_auto_sync,
            auto_remove=auto_remove,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        documentation_id: str,
        *,
        agent_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsGetResponse:
        """
        Get details about a specific documentation making up the agent's knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        agent_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.get(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(documentation_id, agent_id=agent_id, request_options=request_options)
        return _response.data

    async def delete(
        self,
        documentation_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Delete a document or folder from the knowledge base.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        force : typing.Optional[bool]
            If set to true, the document or folder will be deleted regardless of whether it is used by any agents and it will be removed from the dependent agents. For non-empty folders, this will also delete all child documents and folders.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.delete(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
                force=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(documentation_id, force=force, request_options=request_options)
        return _response.data

    async def update(
        self,
        documentation_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsUpdateResponse:
        """
        Update the name and/or content of a document.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        name : typing.Optional[str]
            A custom, human-readable name for the document.

        content : typing.Optional[str]
            Updated content for the document. Only supported for text documents, URL documents with auto-sync disabled, and file documents.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsUpdateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.update(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            documentation_id, name=name, content=content, request_options=request_options
        )
        return _response.data

    async def get_agents(
        self,
        documentation_id: str,
        *,
        dependent_type: typing.Optional[KnowledgeBaseDependentType] = None,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseDependentAgentsResponseModel:
        """
        Get a list of agents depending on this knowledge base document

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        dependent_type : typing.Optional[KnowledgeBaseDependentType]
            Type of dependent agents to return.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseDependentAgentsResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.get_agents(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
                dependent_type="direct",
                page_size=1,
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_agents(
            documentation_id,
            dependent_type=dependent_type,
            page_size=page_size,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    async def get_content(
        self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Get the entire content of a document from the knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Streaming document content

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.get_content(
                documentation_id="documentation_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_content(documentation_id, request_options=request_options)
        return _response.data

    async def get_source_file_url(
        self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> KnowledgeBaseSourceFileUrlResponseModel:
        """
        Get a signed URL to download the original source file of a file-type document from the knowledge base

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseSourceFileUrlResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.get_source_file_url(
                documentation_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_source_file_url(documentation_id, request_options=request_options)
        return _response.data

    async def move(
        self,
        document_id: str,
        *,
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Moves the entity from one folder to another.

        Parameters
        ----------
        document_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.move(
                document_id="21m00Tcm4TlvDq8ikWAM",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.move(document_id, move_to=move_to, request_options=request_options)
        return _response.data

    async def bulk_move(
        self,
        *,
        document_ids: typing.Sequence[str],
        move_to: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Moves multiple entities from one folder to another.

        Parameters
        ----------
        document_ids : typing.Sequence[str]
            The ids of documents or folders from the knowledge base.

        move_to : typing.Optional[str]
            The folder to move the entities to. If not set, the entities will be moved to the root folder.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.documents.bulk_move(
                document_ids=["21m00Tcm4TlvDq8ikWAM", "31m00Tcm4TlvDq8ikWBM"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_move(
            document_ids=document_ids, move_to=move_to, request_options=request_options
        )
        return _response.data

    @property
    def summaries(self):
        if self._summaries is None:
            from .summaries.client import AsyncSummariesClient  # noqa: E402

            self._summaries = AsyncSummariesClient(client_wrapper=self._client_wrapper)
        return self._summaries

    @property
    def chunk(self):
        if self._chunk is None:
            from .chunk.client import AsyncChunkClient  # noqa: E402

            self._chunk = AsyncChunkClient(client_wrapper=self._client_wrapper)
        return self._chunk

    @property
    def chunks(self):
        if self._chunks is None:
            from .chunks.client import AsyncChunksClient  # noqa: E402

            self._chunks = AsyncChunksClient(client_wrapper=self._client_wrapper)
        return self._chunks



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from ...types.environment_variable_auth_connection_value_request import EnvironmentVariableAuthConnectionValueRequest
from ...types.environment_variable_secret_value_request import EnvironmentVariableSecretValueRequest


class EnvironmentVariablesCreateRequestBody_String(UncheckedBaseModel):
    type: typing.Literal["string"] = "string"
    label: str
    values: typing.Dict[str, str]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EnvironmentVariablesCreateRequestBody_Secret(UncheckedBaseModel):
    type: typing.Literal["secret"] = "secret"
    label: str
    values: typing.Dict[str, EnvironmentVariableSecretValueRequest]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class EnvironmentVariablesCreateRequestBody_AuthConnection(UncheckedBaseModel):
    type: typing.Literal["auth_connection"] = "auth_connection"
    label: str
    values: typing.Dict[str, EnvironmentVariableAuthConnectionValueRequest]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


EnvironmentVariablesCreateRequestBody = typing_extensions.Annotated[
    typing.Union[
        EnvironmentVariablesCreateRequestBody_String,
        EnvironmentVariablesCreateRequestBody_Secret,
        EnvironmentVariablesCreateRequestBody_AuthConnection,
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .pronunciation_dictionary_locator import PronunciationDictionaryLocator
from .text_to_dialogue_websocket_voice_input import TextToDialogueWebsocketVoiceInput
from .text_to_dialogue_websocket_voice_settings import TextToDialogueWebsocketVoiceSettings


class TextToDialogueWebsocketClientMessage(UncheckedBaseModel):
    """
    All fields are optional unless noted for the **first** message.

    **First message requirements**
    - `voices`: non-empty array of voice IDs (maximum 10 for `eleven_v3`; exactly 1 for `eleven_v3_conversational`).
    - Credentials if not supplied via `xi-api-key` / `Authorization` headers or `single_use_token` query parameter.

    **Subsequent messages**
    - Do not resend `voices`, `pronunciation_dictionary_locators`, or credential fields.
    """

    inputs: typing.Optional[typing.List[TextToDialogueWebsocketVoiceInput]] = pydantic.Field(default=None)
    """
    Dialogue lines to append for synthesis.
    """

    flush: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Force generation of buffered text without closing the connection.
    """

    close_socket: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Flush buffers, emit remaining audio, send `is_final`, and close the WebSocket.
    """

    keep_alive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resets the 20s inactivity timer; performs no synthesis.
    """

    xi_api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    API key for the first message if not provided via the `xi-api-key` header.
    """

    authorization: typing.Optional[str] = pydantic.Field(default=None)
    """
    Bearer token for the first message if not provided via the `Authorization` header.
    """

    single_use_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    Single-use token for the first message if not provided via the `single_use_token` query parameter.
    """

    voices: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Voice IDs to load for the session (first message only, required on first message).
    """

    voice_settings: typing.Optional[TextToDialogueWebsocketVoiceSettings] = pydantic.Field(default=None)
    """
    Optional voice settings (first message only).
    """

    pronunciation_dictionary_locators: typing.Optional[typing.List[PronunciationDictionaryLocator]] = pydantic.Field(
        default=None
    )
    """
    Optional pronunciation dictionaries (first message only).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

GenesysRegion = typing.Union[
    typing.Literal[
        "us_east_1",
        "eu_west_1",
        "ap_southeast_2",
        "ap_northeast_1",
        "eu_central_1",
        "us_west_2",
        "ca_central_1",
        "ap_northeast_2",
        "eu_west_2",
        "ap_south_1",
        "us_east_2",
        "sa_east_1",
        "me_central_1",
        "ap_northeast_3",
        "eu_central_2",
        "mx_central_1",
        "ap_southeast_1",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .system_tool_config_output import SystemToolConfigOutput


class BuiltInToolsWorkflowOverrideOutput(UncheckedBaseModel):
    end_call: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The end call tool
    """

    language_detection: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The language detection tool
    """

    transfer_to_agent: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The transfer to agent tool
    """

    transfer_to_number: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The transfer to number tool
    """

    skip_turn: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The skip turn tool
    """

    play_keypad_touch_tone: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The play DTMF tool
    """

    voicemail_detection: typing.Optional[SystemToolConfigOutput] = pydantic.Field(default=None)
    """
    The voicemail detection tool
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .conversation_history_transcript_api_integration_webhook_tools_result_common_model_output import (
    ConversationHistoryTranscriptApiIntegrationWebhookToolsResultCommonModelOutput,
)
from .conversation_history_transcript_other_tools_result_common_model import (
    ConversationHistoryTranscriptOtherToolsResultCommonModel,
)
from .conversation_history_transcript_system_tool_result_common_model_output import (
    ConversationHistoryTranscriptSystemToolResultCommonModelOutput,
)

if typing.TYPE_CHECKING:
    from .conversation_history_transcript_workflow_tools_result_common_model_output import (
        ConversationHistoryTranscriptWorkflowToolsResultCommonModelOutput,
    )
WorkflowToolNestedToolsStepModelOutputResultsItem = typing.Union[
    ConversationHistoryTranscriptOtherToolsResultCommonModel,
    ConversationHistoryTranscriptSystemToolResultCommonModelOutput,
    ConversationHistoryTranscriptApiIntegrationWebhookToolsResultCommonModelOutput,
    "ConversationHistoryTranscriptWorkflowToolsResultCommonModelOutput",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .external_sync_type import ExternalSyncType


class ExternalFolderSyncInfo(UncheckedBaseModel):
    """
    Metadata for a KB folder that mirrors an external source folder.
    """

    type: ExternalSyncType = pydantic.Field(default="google_drive")
    """
    Provider identifier
    """

    source_entity_id: str = pydantic.Field()
    """
    Entity ID in the external system
    """

    integration_connection_id: str = pydantic.Field()
    """
    Integration connection instance ID
    """

    root_folder_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    KB folder ID of the sync root. None means this folder is the root.
    """

    sync_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Opaque cursor for incremental sync, interpreted by the provider
    """

    last_sync_at: typing.Optional[int] = pydantic.Field(default=None)
    """
    Unix timestamp of last completed sync
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.workspace_audit_logs_page_response import WorkspaceAuditLogsPageResponse
from .raw_client import AsyncRawAuditLogsClient, RawAuditLogsClient


class AuditLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuditLogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuditLogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuditLogsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        time_from_unix_ms: typing.Optional[int] = None,
        time_to_unix_ms: typing.Optional[int] = None,
        actor_uid: typing.Optional[str] = None,
        class_name: typing.Optional[str] = None,
        activity_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceAuditLogsPageResponse:
        """
        Returns the audit log for the workspace. Requires enterprise tier and the audit_log_read permission.

        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entries per page

        cursor : typing.Optional[str]
            Cursor for the next page (from previous response)

        time_from_unix_ms : typing.Optional[int]
            Only include entries at or after this time (ms since epoch)

        time_to_unix_ms : typing.Optional[int]
            Only include entries at or before this time (ms since epoch)

        actor_uid : typing.Optional[str]
            Filter by actor user ID

        class_name : typing.Optional[str]
            Filter by OCSF event class name (e.g. Account Change)

        activity_name : typing.Optional[str]
            Filter by audit activity name (e.g. Subscription Creation)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceAuditLogsPageResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.audit_logs.list(
            limit=1,
            cursor="cursor",
            time_from_unix_ms=1,
            time_to_unix_ms=1,
            actor_uid="actor_uid",
            class_name="class_name",
            activity_name="activity_name",
        )
        """
        _response = self._raw_client.list(
            limit=limit,
            cursor=cursor,
            time_from_unix_ms=time_from_unix_ms,
            time_to_unix_ms=time_to_unix_ms,
            actor_uid=actor_uid,
            class_name=class_name,
            activity_name=activity_name,
            request_options=request_options,
        )
        return _response.data


class AsyncAuditLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuditLogsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuditLogsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuditLogsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        time_from_unix_ms: typing.Optional[int] = None,
        time_to_unix_ms: typing.Optional[int] = None,
        actor_uid: typing.Optional[str] = None,
        class_name: typing.Optional[str] = None,
        activity_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceAuditLogsPageResponse:
        """
        Returns the audit log for the workspace. Requires enterprise tier and the audit_log_read permission.

        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entries per page

        cursor : typing.Optional[str]
            Cursor for the next page (from previous response)

        time_from_unix_ms : typing.Optional[int]
            Only include entries at or after this time (ms since epoch)

        time_to_unix_ms : typing.Optional[int]
            Only include entries at or before this time (ms since epoch)

        actor_uid : typing.Optional[str]
            Filter by actor user ID

        class_name : typing.Optional[str]
            Filter by OCSF event class name (e.g. Account Change)

        activity_name : typing.Optional[str]
            Filter by audit activity name (e.g. Subscription Creation)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceAuditLogsPageResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.audit_logs.list(
                limit=1,
                cursor="cursor",
                time_from_unix_ms=1,
                time_to_unix_ms=1,
                actor_uid="actor_uid",
                class_name="class_name",
                activity_name="activity_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            limit=limit,
            cursor=cursor,
            time_from_unix_ms=time_from_unix_ms,
            time_to_unix_ms=time_to_unix_ms,
            actor_uid=actor_uid,
            class_name=class_name,
            activity_name=activity_name,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from .....core.pydantic_utilities import IS_PYDANTIC_V2
from .....core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .....types.agent_summary_response_model import AgentSummaryResponseModel


class SummariesGetResponseValue_Success(UncheckedBaseModel):
    status: typing.Literal["success"] = "success"
    data: AgentSummaryResponseModel

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class SummariesGetResponseValue_Failure(UncheckedBaseModel):
    status: typing.Literal["failure"] = "failure"
    error_code: int
    error_status: str
    error_message: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


SummariesGetResponseValue = typing_extensions.Annotated[
    typing.Union[SummariesGetResponseValue_Success, SummariesGetResponseValue_Failure],
    UnionMetadata(discriminant="status"),
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .environment_variables_create_request_body import (
        EnvironmentVariablesCreateRequestBody,
        EnvironmentVariablesCreateRequestBody_AuthConnection,
        EnvironmentVariablesCreateRequestBody_Secret,
        EnvironmentVariablesCreateRequestBody_String,
    )
    from .environment_variables_list_request_type import EnvironmentVariablesListRequestType
    from .update_environment_variable_request_values_value import UpdateEnvironmentVariableRequestValuesValue
_dynamic_imports: typing.Dict[str, str] = {
    "EnvironmentVariablesCreateRequestBody": ".environment_variables_create_request_body",
    "EnvironmentVariablesCreateRequestBody_AuthConnection": ".environment_variables_create_request_body",
    "EnvironmentVariablesCreateRequestBody_Secret": ".environment_variables_create_request_body",
    "EnvironmentVariablesCreateRequestBody_String": ".environment_variables_create_request_body",
    "EnvironmentVariablesListRequestType": ".environment_variables_list_request_type",
    "UpdateEnvironmentVariableRequestValuesValue": ".update_environment_variable_request_values_value",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "EnvironmentVariablesCreateRequestBody",
    "EnvironmentVariablesCreateRequestBody_AuthConnection",
    "EnvironmentVariablesCreateRequestBody_Secret",
    "EnvironmentVariablesCreateRequestBody_String",
    "EnvironmentVariablesListRequestType",
    "UpdateEnvironmentVariableRequestValuesValue",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class CreateOAuth2ClientCredsRequest(UncheckedBaseModel):
    """
    Request model for creating auth connections - inherits common settings and includes sensitive fields
    """

    name: str
    provider: str
    client_id: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    extra_params: typing.Optional[typing.Dict[str, str]] = None
    basic_auth_in_header: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If True, send client credentials in Authorization header as Basic Auth instead of request body
    """

    client_secret: str
    custom_headers: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .speech_history_item_response import SpeechHistoryItemResponse


class GetSpeechHistoryResponse(UncheckedBaseModel):
    history: typing.List[SpeechHistoryItemResponse] = pydantic.Field()
    """
    A list of speech history items.
    """

    last_history_item_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the last history item.
    """

    has_more: bool = pydantic.Field()
    """
    Whether there are more history items to fetch.
    """

    scanned_until: typing.Optional[int] = pydantic.Field(default=None)
    """
    The timestamp of the last history item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
